{*
Copyright (c) 2007 Cary Jensen. All rights reserved.

This code sample is associated with the presentation
"Building Abstract Data Layers for ASP.NET Applications"
by Cary Jensen, Jensen Data Systems, Inc.

presented at CodeRage 2007, Tuesday, March 13 2007


This code sample is provided for demonstration purposes only

No guarantees or warranties are expressed or implied concerning
the applicability of techniques or code included in this code
sample. If you wish to use techniques or code included in this
code sample, it is your responsibility to test and certify any
code, techniques, or designs adopted as a result of this code sample

For information about training or consulting services
for all versions of Delphi, please visit
http://www.JensenDataSystems.com


*}
unit editorder;

interface

uses
  System.Collections, System.ComponentModel,
  System.Data, System.Drawing, System.Web, System.Web.SessionState,
  System.Web.UI, System.Web.UI.WebControls,  System.Web.UI.HtmlControls,
  JDSI.Modules.SqlDataModule,
  JDSI.Modules.OrderObject;

type
  TEditOrder = class(System.Web.UI.Page)
  {$REGION 'Designer Managed Code'}
  strict private
    procedure InitializeComponent;
    procedure TCustEdit_Unload(sender: System.Object; e: System.EventArgs);
    procedure Button2_Click(sender: System.Object; e: System.EventArgs);
    procedure Button1_Click(sender: System.Object; e: System.EventArgs);
  {$ENDREGION}
  strict private
    procedure Page_Load(sender: System.Object; e: System.EventArgs);
  strict protected
    ShipNameTextBox: System.Web.UI.WebControls.TextBox;
    ShipAddressTextBox: System.Web.UI.WebControls.TextBox;
    ShipCityTextBox: System.Web.UI.WebControls.TextBox;
    ShipRegionTextBox: System.Web.UI.WebControls.TextBox;
    ShipCountryTextBox: System.Web.UI.WebControls.TextBox;
    SaveButton: System.Web.UI.WebControls.Button;
    Button2: System.Web.UI.WebControls.Button;
    ErrorMessageLabel: System.Web.UI.WebControls.Label;
    procedure OnInit(e: EventArgs); override;
  private
    { Private Declarations }
    SqlDataModule: TSqlDataModule;
    OrderID: String;
    procedure SetErrorMessage(Value: String);
  public
    { Public Declarations }
  end;

implementation

{$REGION 'Designer Managed Code'}
/// <summary>
/// Required method for Designer support -- do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TEditOrder.InitializeComponent;
begin
  Include(Self.SaveButton.Click, Self.Button1_Click);
  Include(Self.Button2.Click, Self.Button2_Click);
  Include(Self.Unload, Self.TCustEdit_Unload);
  Include(Self.Load, Self.Page_Load);
end;
{$ENDREGION}

procedure TEditOrder.Page_Load(sender: System.Object; e: System.EventArgs);
var
  OrderObject: TOrderObject;
begin
  // TODO: Put user code to initialize the page here
  if not IsPostBack then
  begin
    OrderObject := TOrderObject.Create(SqlDataModule);
    OrderObject.LoadData(OrderID);
    ShipNameTextBox.Text := OrderObject.ShipName;
    ShipAddressTextBox.Text := OrderObject.ShipAddress;
    ShipCityTextBox.Text := OrderObject.ShipCity;
    ShipRegionTextBox.Text := OrderObject.ShipRegion;
    ShipCountryTextBox.Text := OrderObject.ShipCountry;
  end;
end;

procedure TEditOrder.SetErrorMessage(Value: String);
begin
  ErrorMessageLabel.Text := Value;
end;

procedure TEditOrder.OnInit(e: EventArgs);
begin
  //
  // Required for Designer support
  //
  SqlDataModule:= TSqlDataModule.Create(Request, Response, Session, Trace);
  SqlDataModule.VerifyLogin;
  OrderID := Request.QueryString.Item['orderid'];
  InitializeComponent;
  inherited OnInit(e);
end;

procedure TEditOrder.Button1_Click(sender: System.Object; e: System.EventArgs);
var
  OrderObject: TOrderObject;
begin
  try
    OrderObject := TOrderObject.Create(SqlDataModule);
    OrderObject.ShipName := ShipNameTextBox.Text;
    OrderObject.ShipAddress := ShipAddressTextBox.Text;
    OrderObject.ShipCity := ShipCityTextBox.Text;
    OrderObject.ShipRegion := ShipRegionTextBox.Text;
    OrderObject.ShipCountry := ShipCountryTextBox.Text;
    OrderObject.Validate;
    if not OrderObject.IsValid then
    begin
      SetErrorMessage(OrderObject.ErrorMessage);
      Exit;
    end;
    OrderObject.SaveData(OrderID);
  except
    on e: Exception do
    begin
      SetErrorMessage('Save error: ' + e.Message);
      Exit;
    end;
  end;
  Response.Redirect('DataForm.aspx');
end;

procedure TEditOrder.Button2_Click(sender: System.Object; e: System.EventArgs);
begin
  Response.Redirect('DataForm.aspx');
end;

procedure TEditOrder.TCustEdit_Unload(sender: System.Object; e: System.EventArgs);
begin
  SqlDataModule.Dispose;
end;

end.

